pg.ddx.io  pgsql-performance@postgresql.org mailing list archive  
help / color / mirror / Atom feed
From: Miernik <public@public.miernik.name>
To: pgsql-performance@postgresql.org
Subject: Re: why query plan for the inner SELECT of WHERE x IN is wrong, but when run the inner query alone is OK?
Date: Sun, 10 Aug 2008 04:20:59 +0200
Message-ID: <20080810022059.7330.0.NOFFLE@turbacz.local> (raw)
References: <20080809203435.7227.0.NOFFLE@turbacz.local>
	<20080809223208.7297.0.NOFFLE@turbacz.local>

Miernik <public@public.miernik.name> wrote:
> I present a SELECT uid plan with the 1000 table also below, just to be
> sure, this is the "bad" plan, that takes forever:
> 
> miernik=> EXPLAIN SELECT uid FROM cnts WHERE uid IN (SELECT uid FROM alog WHERE pid = 3452654 AND o = 1);
>                                          QUERY PLAN
> -----------------------------------------------------------------------------------------------
> Nested Loop IN Join  (cost=0.00..3532.70 rows=1 width=4)
>   ->  Seq Scan on cnts  (cost=0.00..26.26 rows=1026 width=4)
>   ->  Index Scan using alog_uid_idx on alog  (cost=0.00..297.32 rows=1 width=4)
>         Index Cond: ((alog.uid)::integer = (cnts.uid)::integer)
>         Filter: ((alog.pid = 3452654::numeric) AND (alog.o = 1::numeric))
> (5 rows)

If I reduce the number of rows in cnts to 100, I can actually make an
EXPLAIN ANALYZE with this query plan finish in reasonable time:

miernik=> EXPLAIN ANALYZE SELECT uid FROM cnts WHERE uid IN (SELECT uid FROM alog WHERE pid = 555949 AND odp = 1);
                                                                   QUERY PLAN
-------------------------------------------------------------------------------------------------------------------------------------------------
 Nested Loop IN Join  (cost=0.00..3585.54 rows=1 width=4) (actual time=51831.430..267844.815 rows=7 loops=1)
   ->  Seq Scan on cnts  (cost=0.00..14.00 rows=700 width=4) (actual time=0.005..148.464 rows=100 loops=1)
   ->  Index Scan using alog_uid_idx on alog  (cost=0.00..301.02 rows=1 width=4) (actual time=2676.959..2676.959 rows=0 loops=100)
         Index Cond: ((alog.uid)::integer = (cnts.uid)::integer)
         Filter: ((alog.pid = 555949::numeric) AND (alog.odp = 1::numeric))
 Total runtime: 267844.942 ms
(6 rows)

The real running times are about 10 times more than the estimates. Is
that normal?

-- 
Miernik
http://miernik.name/




view thread (9+ messages)  latest in thread

Message-ID: <20080810022059.7330.0.NOFFLE@turbacz.local>
Permalink:  ../20080810022059.7330.0.NOFFLE@turbacz.local/
Also on:    postgresql.org/message-id/20080810022059.7330.0.NOFFLE@turbacz.local

 · 

reply

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Reply to all the recipients using the --to and --cc options:
  reply via email

  To: pgsql-performance@postgresql.org
  Cc: public@public.miernik.name
  Subject: Re: why query plan for the inner SELECT of WHERE x IN is wrong, but when run the inner query alone is OK?
  In-Reply-To: <20080810022059.7330.0.NOFFLE@turbacz.local>

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

This inbox is served by DDX for PostgreSQL; see mirroring instructions
for how to clone and mirror all data and code used for this inbox